/**
 * @file   compat/platform_spin.c
 * @author Martin Pulec     <pulec@cesnet.cz>
 */
/*
 * Copyright (c) 2013 CESNET z.s.p.o.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of CESNET nor the names of its contributors may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "compat/platform_spin.h"

#include "debug.h"

#ifdef HAVE_MACOSX
int platform_spin_init(platform_spin_t *spin)
{
        *spin = OS_SPINLOCK_INIT;
        return 0;
}

int platform_spin_lock(platform_spin_t *spin)
{
        OSSpinLockLock(spin);
        return 0;
}

int platform_spin_unlock(platform_spin_t *spin)
{
        OSSpinLockUnlock(spin);
        return 0;
}

int platform_spin_destroy(platform_spin_t *spin)
{
        UNUSED(spin);
        return 0;
}
#else
int platform_spin_init(platform_spin_t *spin)
{
        return pthread_spin_init(spin, 0);
}

int platform_spin_lock(platform_spin_t *spin)
{
        return pthread_spin_lock(spin);
}

int platform_spin_unlock(platform_spin_t *spin)
{
        return pthread_spin_unlock(spin);
}

int platform_spin_destroy(platform_spin_t *spin)
{
        return pthread_spin_destroy(spin);
}
#endif

