/*
 * FILE:    test_des.c
 * AUTHORS: Colin Perkins
 *
 * The test vectors and outline structure of this are taken from Eric
 * Young's destest.c from libdes, version 3.00 (7 October 1993). 
 *
 * Copyright (c) 2003-2004 University of Southern California 
 * Copyright (c) 1999-2000 University College London
 * Copyright (c) 1993 Eric Young
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions 
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Computer Science
 *      Department at University College London
 * 4. Neither the name of the University nor of the Department may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "config.h"
#include "config_unix.h"
#include "config_win32.h"
#include "debug.h"
#include "crypto/crypt_des.h"
#include "test_des.h"

#define NUM_TESTS 36

static unsigned char key_data[NUM_TESTS][8] = {
        {0x12, 0x34, 0x57, 0x79, 0x9b, 0xbc, 0xdf, 0xf1},
        {0x0e, 0x32, 0x92, 0x32, 0xea, 0x6d, 0x0d, 0x72},
        {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
        {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
        {0x30, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
        {0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11},
        {0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF},
        {0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11},
        {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
        {0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10},
        {0x7C, 0xA1, 0x10, 0x45, 0x4A, 0x1A, 0x6E, 0x57},
        {0x01, 0x31, 0xD9, 0x61, 0x9D, 0xC1, 0x37, 0x6E},
        {0x07, 0xA1, 0x13, 0x3E, 0x4A, 0x0B, 0x26, 0x86},
        {0x38, 0x49, 0x67, 0x4C, 0x26, 0x02, 0x31, 0x9E},
        {0x04, 0xB9, 0x15, 0xBA, 0x43, 0xFE, 0xB5, 0xB6},
        {0x01, 0x13, 0xB9, 0x70, 0xFD, 0x34, 0xF2, 0xCE},
        {0x01, 0x70, 0xF1, 0x75, 0x46, 0x8F, 0xB5, 0xE6},
        {0x43, 0x29, 0x7F, 0xAD, 0x38, 0xE3, 0x73, 0xFE},
        {0x07, 0xA7, 0x13, 0x70, 0x45, 0xDA, 0x2A, 0x16},
        {0x04, 0x68, 0x91, 0x04, 0xC2, 0xFD, 0x3B, 0x2F},
        {0x37, 0xD0, 0x6B, 0xB5, 0x16, 0xCB, 0x75, 0x46},
        {0x1F, 0x08, 0x26, 0x0D, 0x1A, 0xC2, 0x46, 0x5E},
        {0x58, 0x40, 0x23, 0x64, 0x1A, 0xBA, 0x61, 0x76},
        {0x02, 0x58, 0x16, 0x16, 0x46, 0x29, 0xB0, 0x07},
        {0x49, 0x79, 0x3E, 0xBC, 0x79, 0xB3, 0x25, 0x8F},
        {0x4F, 0xB0, 0x5E, 0x15, 0x15, 0xAB, 0x73, 0xA7},
        {0x49, 0xE9, 0x5D, 0x6D, 0x4C, 0xA2, 0x29, 0xBF},
        {0x01, 0x83, 0x10, 0xDC, 0x40, 0x9B, 0x26, 0xD6},
        {0x1C, 0x58, 0x7F, 0x1C, 0x13, 0x92, 0x4F, 0xEF},
        {0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01},
        {0x1F, 0x1F, 0x1F, 0x1F, 0x0E, 0x0E, 0x0E, 0x0E},
        {0xE0, 0xFE, 0xE0, 0xFE, 0xF1, 0xFE, 0xF1, 0xFE},
        {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
        {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
        {0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF},
        {0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10}
};

static unsigned char plain_data[NUM_TESTS][8] = {
        {0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef},
        {0x87, 0x87, 0x87, 0x87, 0x87, 0x87, 0x87, 0x87},
        {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
        {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
        {0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01},
        {0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11},
        {0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11},
        {0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF},
        {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
        {0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF},
        {0x01, 0xA1, 0xD6, 0xD0, 0x39, 0x77, 0x67, 0x42},
        {0x5C, 0xD5, 0x4C, 0xA8, 0x3D, 0xEF, 0x57, 0xDA},
        {0x02, 0x48, 0xD4, 0x38, 0x06, 0xF6, 0x71, 0x72},
        {0x51, 0x45, 0x4B, 0x58, 0x2D, 0xDF, 0x44, 0x0A},
        {0x42, 0xFD, 0x44, 0x30, 0x59, 0x57, 0x7F, 0xA2},
        {0x05, 0x9B, 0x5E, 0x08, 0x51, 0xCF, 0x14, 0x3A},
        {0x07, 0x56, 0xD8, 0xE0, 0x77, 0x47, 0x61, 0xD2},
        {0x76, 0x25, 0x14, 0xB8, 0x29, 0xBF, 0x48, 0x6A},
        {0x3B, 0xDD, 0x11, 0x90, 0x49, 0x37, 0x28, 0x02},
        {0x26, 0x95, 0x5F, 0x68, 0x35, 0xAF, 0x60, 0x9A},
        {0x16, 0x4D, 0x5E, 0x40, 0x4F, 0x27, 0x52, 0x32},
        {0x6B, 0x05, 0x6E, 0x18, 0x75, 0x9F, 0x5C, 0xCA},
        {0x00, 0x4B, 0xD6, 0xEF, 0x09, 0x17, 0x60, 0x62},
        {0x48, 0x0D, 0x39, 0x00, 0x6E, 0xE7, 0x62, 0xF2},
        {0x43, 0x75, 0x40, 0xC8, 0x69, 0x8F, 0x3C, 0xFA},
        {0x07, 0x2D, 0x43, 0xA0, 0x77, 0x07, 0x52, 0x92},
        {0x02, 0xFE, 0x55, 0x77, 0x81, 0x17, 0xF1, 0x2A},
        {0x1D, 0x9D, 0x5C, 0x50, 0x18, 0xF7, 0x28, 0xC2},
        {0x30, 0x55, 0x32, 0x28, 0x6D, 0x6F, 0x29, 0x5A},
        {0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF},
        {0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF},
        {0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF},
        {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
        {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
        {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
        {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}
};

static unsigned char crypt_data[NUM_TESTS][8] = {
        {0x85, 0xe8, 0x13, 0x54, 0x0f, 0x0a, 0xb4, 0x05},
        {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
        {0x8C, 0xA6, 0x4D, 0xE9, 0xC1, 0xB1, 0x23, 0xA7},
        {0x73, 0x59, 0xB2, 0x16, 0x3E, 0x4E, 0xDC, 0x58},
        {0x95, 0x8E, 0x6E, 0x62, 0x7A, 0x05, 0x55, 0x7B},
        {0xF4, 0x03, 0x79, 0xAB, 0x9E, 0x0E, 0xC5, 0x33},
        {0x17, 0x66, 0x8D, 0xFC, 0x72, 0x92, 0x53, 0x2D},
        {0x8A, 0x5A, 0xE1, 0xF8, 0x1A, 0xB8, 0xF2, 0xDD},
        {0x8C, 0xA6, 0x4D, 0xE9, 0xC1, 0xB1, 0x23, 0xA7},
        {0xED, 0x39, 0xD9, 0x50, 0xFA, 0x74, 0xBC, 0xC4},
        {0x69, 0x0F, 0x5B, 0x0D, 0x9A, 0x26, 0x93, 0x9B},
        {0x7A, 0x38, 0x9D, 0x10, 0x35, 0x4B, 0xD2, 0x71},
        {0x86, 0x8E, 0xBB, 0x51, 0xCA, 0xB4, 0x59, 0x9A},
        {0x71, 0x78, 0x87, 0x6E, 0x01, 0xF1, 0x9B, 0x2A},
        {0xAF, 0x37, 0xFB, 0x42, 0x1F, 0x8C, 0x40, 0x95},
        {0x86, 0xA5, 0x60, 0xF1, 0x0E, 0xC6, 0xD8, 0x5B},
        {0x0C, 0xD3, 0xDA, 0x02, 0x00, 0x21, 0xDC, 0x09},
        {0xEA, 0x67, 0x6B, 0x2C, 0xB7, 0xDB, 0x2B, 0x7A},
        {0xDF, 0xD6, 0x4A, 0x81, 0x5C, 0xAF, 0x1A, 0x0F},
        {0x5C, 0x51, 0x3C, 0x9C, 0x48, 0x86, 0xC0, 0x88},
        {0x0A, 0x2A, 0xEE, 0xAE, 0x3F, 0xF4, 0xAB, 0x77},
        {0xEF, 0x1B, 0xF0, 0x3E, 0x5D, 0xFA, 0x57, 0x5A},
        {0x88, 0xBF, 0x0D, 0xB6, 0xD7, 0x0D, 0xEE, 0x56},
        {0xA1, 0xF9, 0x91, 0x55, 0x41, 0x02, 0x0B, 0x56},
        {0x6F, 0xBF, 0x1C, 0xAF, 0xCF, 0xFD, 0x05, 0x56},
        {0x2F, 0x22, 0xE4, 0x9B, 0xAB, 0x7C, 0xA1, 0xAC},
        {0x5A, 0x6B, 0x61, 0x2C, 0xC2, 0x6C, 0xCE, 0x4A},
        {0x5F, 0x4C, 0x03, 0x8E, 0xD1, 0x2B, 0x2E, 0x41},
        {0x63, 0xFA, 0xC0, 0xD0, 0x34, 0xD9, 0xF7, 0x93},
        {0x61, 0x7B, 0x3A, 0x0C, 0xE8, 0xF0, 0x71, 0x00},
        {0xDB, 0x95, 0x86, 0x05, 0xF8, 0xC8, 0xC6, 0x06},
        {0xED, 0xBF, 0xD1, 0xC6, 0x6C, 0x29, 0xCC, 0xC7},
        {0x35, 0x55, 0x50, 0xB2, 0x15, 0x0E, 0x24, 0x51},
        {0xCA, 0xAA, 0xAF, 0x4D, 0xEA, 0xF1, 0xDB, 0xAE},
        {0xD5, 0xD4, 0x4F, 0xF7, 0x20, 0x68, 0x3D, 0x0D},
        {0x2A, 0x2B, 0xB0, 0x08, 0xDF, 0x97, 0xC2, 0xF2}
};

static unsigned char cbc_key[8] =
    { 0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef };
static unsigned char cbc_iv[8] =
    { 0xfe, 0xdc, 0xba, 0x98, 0x76, 0x54, 0x32, 0x10 };
static unsigned char cbc_data[32] = "7654321 Now is the time for ";

static unsigned char cbc_ok[32] = {
        0xcc, 0xd1, 0x73, 0xff, 0xab, 0x20, 0x39, 0xf4,
        0xac, 0xd8, 0xae, 0xfd, 0xdf, 0xd8, 0xa1, 0xeb,
        0x46, 0x8e, 0x91, 0x15, 0x78, 0x88, 0xba, 0x68,
        0x1d, 0x26, 0x93, 0x97, 0xf7, 0xfe, 0x62, 0xb4
};

int test_des(void)
{
        int i, j;
        unsigned char key[8], plain[8], crypt[8], buffer[8];
        unsigned char cbc_buffer[32], cbc_iv_buf[8];

        printf
            ("Testing DES in ECB mode .................................................. ");
        fflush(stdout);
        for (i = 0; i < NUM_TESTS; i++) {
                for (j = 0; j < 8; j++) {
                        key[j] = key_data[i][j];
                        plain[j] = plain_data[i][j];
                        crypt[j] = crypt_data[i][j];
                }
                memcpy(buffer, crypt, 8);
                qfDES_ECB_d(key, buffer, 8);
                if (memcmp(buffer, plain, 8) != 0) {
                        printf("FAIL\n");
                        goto abort_ecb;
                }
                memcpy(buffer, plain, 8);
                qfDES_ECB_e(key, buffer, 8);
                if (memcmp(buffer, crypt, 8) != 0) {
                        printf("FAIL\n");
                        goto abort_ecb;
                }
                qfDES_ECB_d(key, buffer, 8);
                if (memcmp(buffer, plain, 8) != 0) {
                        printf("FAIL\n");
                        goto abort_ecb;
                }
        }
        printf("Ok\n");
 abort_ecb:

        printf
            ("Testing DES in CBC mode .................................................. ");
        fflush(stdout);
        memcpy(cbc_buffer, cbc_data, 32);
        memcpy(cbc_iv_buf, cbc_iv, 8);
        qfDES_CBC_e(cbc_key, cbc_buffer, 32, cbc_iv_buf);
        if (memcmp(cbc_buffer, cbc_ok, 32) != 0) {
                printf("FAIL\n");
                return 1;
        }
        memcpy(cbc_iv_buf, cbc_iv, 8);
        qfDES_CBC_d(cbc_key, cbc_buffer, 32, cbc_iv_buf);
        if (memcmp(cbc_buffer, cbc_data, 32) != 0) {
                printf("FAIL\n");
                return 1;
        }
        printf("Ok\n");
        return 0;
}
